﻿// Prism Demo (Prism) //Note: This class is heavily based on other Papervision3D object classes 
//
// released under MIT License
// http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.0.0
//@website 		http://www.permaven.com, http://www.almostlogical.com



//Copyright (c) 2008 Devin Reimer
	
package com
{
	import org.papervision3d.Papervision3D;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.materials.utils.MaterialsList;	
	import org.papervision3d.core.proto.*;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.primitives.Cylinder;

	import com.Triangle;
	 
	 
	public class Prism extends DisplayObject3D
	{
		 /**
		* No faces selected.
		*/
		static public var NONE   :int = 0x00;
		/**
		* Front face selection
		*/
		static public var FRONT  :int = 0x01;
	
		/**
		* Right face selection
		*/
		static public var RIGHT  :int = 0x04;
	
		/**
		* Left face selection
		*/
		static public var LEFT   :int = 0x08;
	
		/**
		* Top face selection
		*/
		static public var TOP    :int = 0x10;
	
		/**
		* Bottom face selection
		*/
		static public var BOTTOM :int = 0x20;
	
		/**
		* All faces selected.
		*/
		static public var ALL    :int = FRONT + RIGHT + LEFT + TOP + BOTTOM;
		
		///public var allMaterials:MaterialsList;
		
		
		var frontPlane:Plane;
		var leftPlane:Plane;
		var rightPlane:Plane;
		var topTriangle1:Triangle;
		var topTriangle2:Triangle;
		var bottomTriangle1:Triangle;
		var bottomTriangle2:Triangle;
		
		
		public function Prism(theMaterials:MaterialsList=null, width:Number=100, height:Number=200,segmentsW:int=2, segmentsH:int=2, isDoubleSided:Boolean=false, isTopAndBottom:Boolean=true) 
		{
			var newHeight:Number;
			var matInstance:MaterialObject3D;
			
			materials=theMaterials;
			
			frontPlane=buildPlane("front",width,height,segmentsW,segmentsH,isDoubleSided);
			frontPlane.rotationY=270;
			frontPlane.x+=width/2.33333;
			
			leftPlane = buildPlane("left",width,height,segmentsW,segmentsH,isDoubleSided);
			leftPlane.z-=width/4;
			leftPlane.rotationY=30;

			rightPlane = buildPlane("right",width,height,segmentsW,segmentsH,isDoubleSided);
			rightPlane.rotationY=150;
			rightPlane.z+=width/4;
			
			addChild(frontPlane);
			addChild(leftPlane);
			addChild(rightPlane);
			
			if (isTopAndBottom)
			{
				newHeight=Math.sin(60*(Math.PI/180))*width;
				
				if( ! (matInstance= materials.getMaterialByName("top")))
				{
					if(!(matInstance=materials.getMaterialByName( "all" ))){
						Papervision3D.log( "Prism: Required material not found in given materials list. Supported materials are: front, back, right, left, top, bottom & all." );
						matInstance=new ColorMaterial(0xFFFFFF);
					}
				}
			
				topTriangle1 = new Triangle(matInstance,width/2,newHeight,1,1);
				topTriangle1.rotationY=270;
				topTriangle1.rotationX=90;
				topTriangle1.rotationZ=180;
				topTriangle1.z+=width/4;
				topTriangle1.y+=height/2-.1;
				
				topTriangle2 = new Triangle(matInstance,newHeight,width/2,1,1);
				topTriangle2.rotationY=0;
				topTriangle2.rotationX=90;
				topTriangle2.rotationZ=180;
				topTriangle2.z-=width/4;
				topTriangle2.y+=height/2-.1;
				
				if( ! (matInstance= materials.getMaterialByName("bottom")))
				{
					if(!(matInstance=materials.getMaterialByName( "all" ))){
						Papervision3D.log( "Prism: Required material not found in given materials list. Supported materials are: front, back, right, left, top, bottom & all." );
						matInstance=new ColorMaterial(0xFFFFFF);
					}
				}
				
				bottomTriangle1 = new Triangle(matInstance,-width/2,-newHeight,1,1);
				bottomTriangle1.rotationY=270;
				bottomTriangle1.rotationX=90;
				bottomTriangle1.rotationZ=0;
				bottomTriangle1.z-=width/4;
				bottomTriangle1.y-=height/2-.1;
				
				bottomTriangle2 = new Triangle(matInstance,-newHeight,-width/2,1,1);
				bottomTriangle2.rotationY=0;
				bottomTriangle2.rotationX=90;
				bottomTriangle2.rotationZ=0;
				bottomTriangle2.z+=width/4;
				bottomTriangle2.y-=height/2-.1;
				
				
				addChild(topTriangle1);
				addChild(topTriangle2);
				addChild(bottomTriangle1);
				addChild(bottomTriangle2);
			}
		}
		
		private function buildPlane( mat:String,  width:Number, height:Number, segmentsW:int=2, segmentsH:int=2,reverse:Boolean=false ):Plane
		{
			var thePlane:Plane;
			var matInstance:MaterialObject3D;
			if( ! (matInstance= materials.getMaterialByName( mat )))
			{
				if(!(matInstance=materials.getMaterialByName( "all" ))){
					Papervision3D.log( "Prism: Required material not found in given materials list. Supported materials are: front, back, right, left, top, bottom & all." );
					return thePlane;
				}
			}
			
			thePlane = new Plane(matInstance,width,height,segmentsW,segmentsH);
			if (reverse)
			{
				thePlane.material.doubleSided=true;
			}
			
			return thePlane;
		}
		
		public function editMaterialPlane(planeString:String,newMaterial)
		{
			if (planeString.toLowerCase() == "front" )
			{
				frontPlane.material= newMaterial;
			}
			else if (planeString.toLowerCase() == "left" )
			{
				leftPlane.material=newMaterial;
			}
			else if (planeString.toLowerCase() == "right" )
			{
				rightPlane.material=newMaterial;
			}
			
		}
	}
	
}