﻿// RotatingBillboardDemo - Using Papervision 2.0 - Great White Revision 638
//  Rotating Billboard Effect that uses 4 images, the extra image secretly swaps in an out behind the scenes;
//
// released under MIT License
// http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.0.0
//@website 		http://www.permaven.com, http://www.almostlogical.com


//Copyright (c) 2008 Devin Reimer

/*
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
	
package com
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.display.BitmapData;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import flash.filters.BlurFilter;
	import flash.events.MouseEvent;
	
	import org.papervision3d.cameras.Camera3D;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.materials.BitmapAssetMaterial;
	import org.papervision3d.materials.BitmapMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.render.BasicRenderEngine;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.view.Viewport3D;
	import com.Prism;
	import com.Triangle;
	
	public class RotatingBillboardDemo extends Sprite	
	{
		private var viewport:Viewport3D;
		private var scene:Scene3D;
		private var camera:Camera3D;
		private var renderer:BasicRenderEngine;
		
		private var prismArray:Array;
		private var isMouseEnabled=true;
		private var pauseCounter:Number;
		private var prismRotationCounter:Number;
		
		private var prismSidesArray:Array = new Array();
		
		private var currentFaceString:String;
		private var totalNumberOfImages:int;
		private var nextImageCounter:int;
		
		private const NUM_OF_PRISMS:int = 6;
		private const VERTICAL_SLICES:int = 1;
		
		private const IMAGE_WIDTH:Number = 675;
		private const IMAGE_HEIGHT:Number = 225;
		
		private const PRISM_DEFAULT_WIDTH:Number = 20;
		
		private var centerPrism:Prism;
		
		public function RotatingBillboardDemo()
		{
			stage.quality = "MEDIUM"; //Set the stage quality to Medium (for speed)
			prismArray=new Array();  
			pauseCounter=0;
			currentFaceString="front";
			prismRotationCounter=0;
			initPapervision3D();
			createAllPrisms();
			beginRender();
			
			//addChild(new FPS()); //enable when need Frames Per Second Displayed
		}
		
		public function createAllPrisms()
		{
			var prismQuality:Number=4;
			var cutImageWidth:Number = IMAGE_WIDTH / NUM_OF_PRISMS;
			var cutImageHeight:Number = IMAGE_HEIGHT / VERTICAL_SLICES;
			var prismHeight:Number = PRISM_DEFAULT_WIDTH * (cutImageHeight/cutImageWidth);
			
			var m = new MaterialsList();
			
			//in the future this can be changed to load external images using xml for the image paths, instead of hardcoding
			prismSidesArray[0]=cutUpImage(new Image1(IMAGE_WIDTH,IMAGE_HEIGHT),cutImageWidth,cutImageHeight);
			prismSidesArray[1]=cutUpImage(new Image2(IMAGE_WIDTH,IMAGE_HEIGHT),cutImageWidth,cutImageHeight);
			prismSidesArray[2]=cutUpImage(new Image3(IMAGE_WIDTH,IMAGE_HEIGHT),cutImageWidth,cutImageHeight);
			prismSidesArray[3]=cutUpImage(new Image4(IMAGE_WIDTH,IMAGE_HEIGHT),cutImageWidth,cutImageHeight);
			   
		   
			for (var t=0; t<VERTICAL_SLICES; t++) //used incase you would like to cut into pieces vertical
			{    for (var n=0; n<NUM_OF_PRISMS; n++) 
				{
				   m.addMaterial(new ColorMaterial(0xFFFFFF),"all");
				   var prism = new Prism(m,PRISM_DEFAULT_WIDTH,prismHeight,prismQuality,prismQuality,false);
				   
				   prism.z+=(PRISM_DEFAULT_WIDTH*n)-50;
				   prism.y = 18;
				   scene.addChild(prism);
				   prismArray.push(prism);
				   
				   if (t == Math.ceil(NUM_OF_PRISMS/2))
				   {
					   trace("HERE");
					   centerPrism = prism;
				   }
				}
			}
			
			addCutMaterialToSide("front",prismSidesArray[0]);
			addCutMaterialToSide("right",prismSidesArray[1]);
			addCutMaterialToSide("left",prismSidesArray[2]);
			
			totalNumberOfImages=4; //total number of images
			nextImageCounter=3; //the next image loaded onto the prisms
		}
		
		//This cuts the image up into pieces and this pieces are used to create multiple BitmapMaterials
		// This function returns an array of BitmapMaterials
		public function cutUpImage(imageData:BitmapData,cutImageWidth:Number,cutImageHeight:Number):Array
		{
			var bitmapMaterialArray:Array = new Array();
			var tempBitmapData:BitmapData;
			
			for (var t=0; t<VERTICAL_SLICES; t++) //used incase you would like to cut into pieces vertical
			{    for (var n=0; n<NUM_OF_PRISMS; n++)
				{
					 tempBitmapData = new BitmapData(cutImageWidth,cutImageHeight, false, 0x0000CC44);
					 tempBitmapData.copyPixels(imageData, new Rectangle(cutImageWidth*n,cutImageHeight*t,cutImageWidth,cutImageHeight), new Point(0, 0));
					 bitmapMaterialArray[n*(t+1)] = new BitmapMaterial(tempBitmapData);
				}
			}
			
			return bitmapMaterialArray;
		}
		
		//Adds Bitmap Materials to Prism
		public function addCutMaterialToSide(sideName:String,bitmapMaterialArray:Array)
		{
			for (var t=0; t<VERTICAL_SLICES; t++) //used incase you would like to cut into pieces vertical
			{    for (var n=0; n<NUM_OF_PRISMS; n++)
				{
					prismArray[n*(t+1)].editMaterialPlane(sideName,bitmapMaterialArray[n*(t+1)]);
				}
			} 
		}
		
		private function initPapervision3D():void
		{
		  
			viewport = new Viewport3D();
			addChild(viewport);
			
			scene = new Scene3D();
			camera = new Camera3D();
	
			camera.x=125;

	
			camera.zoom=10;
			camera.focus=100;
			renderer = new BasicRenderEngine(); 
			
		}
		
		private function beginRender():void
		{
		   //calls the render function every frame
			addEventListener(Event.ENTER_FRAME, render);
		}
		
		private function render(e:Event):void
		{
			var prismSpeed=1.5;
		   
			if (prismRotationCounter>=120) //counters current used, timers should be used in the future
			{
				if (pauseCounter<40)
				{
					pauseCounter++;
				}
				else
				{  
					if (totalNumberOfImages>3)
					{
						if (nextImageCounter>=totalNumberOfImages)
						{
							nextImageCounter=0;   
						}
						
						trace(currentFaceString);
						addCutMaterialToSide(currentFaceString,prismSidesArray[nextImageCounter]);
						
						nextImageCounter++;
					}
					
					
					if (currentFaceString == "front")
					{
					   currentFaceString="right";   
					}
					else if (currentFaceString ==  "right")
					{
						currentFaceString="left";   
					}
					else
					{
						currentFaceString="front";   
					}
					
					pauseCounter=0;
					prismRotationCounter=0;   
				}
			}
			else
			{
				prismRotationCounter+=prismSpeed;
				
				for each(var prism in prismArray)
				{
					prism.yaw(prismSpeed);  
				}
			}
		
			
			
			camera.y= (stage.mouseY-(stage.stageHeight/2))/4;
			camera.z = (stage.mouseX-(stage.stageWidth/2))/6;
			
			renderer.renderScene(scene, camera, viewport);
			
		}
    }
}